#----------------------------- DRAG & DROP UTILITY ----------------------------#
# original plugin by IOhannes Zmoelnig, modified by Oliver Stotz
# 
# WHAT IT DOES:
# 
# WHEN A FILE IS DROPPED ONTO THE PD MAIN WINDOW: 
# if it's a .pd file, open it. all other files will be rejected by PD.
# WHEN A FILE IS DROPPED ONTO A PD CANVAS:
# if it's a .pd file, create it on this spot in the canvas
# for all other filetypes: sned a message to recieve object "from_ol_dropfile" - 
# containing srop position and full filename. also create a special object
# called "ol_selfdestruct". this can be "read" by a receivecanvas object, allowing
# drag and drop inside a GOP window only (else it's always global)


package require tkdnd

namespace eval ::dnd_object_create {
    variable x 0
    variable y 0
}

#------------------------------------------------------------------------------#
# create an object using the dropped filename

proc ::dnd_object_create::bind_to_canvas {mytoplevel} {
    ::tkdnd::drop_target register $mytoplevel DND_Files
    bind $mytoplevel <<DropPosition>> {::dnd_object_create::setxy %X %Y}
    bind $mytoplevel <<Drop:DND_Files>> {::dnd_object_create::dropped_object_files %W %D}
}


proc ::dnd_object_create::setxy {newx newy} {
    variable x $newx
    variable y $newy
    return "copy"
}


proc ::dnd_object_create::open_dropped_files {files} {
    foreach file $files {
        open_file $file
   }
}


proc ::dnd_object_create::dropped_object_files {mytoplevel files} {
    foreach file $files {
	set ext  [file extension $file]
	set obj  [file rootname [file tail $file]]
	set dir  [file dirname $file]
#------------------------------------------------------------------------------#	
	set found 0
#------------------------------------------------------------------------------#
    if {$ext == ".pd"} {
        set found 2
        foreach pathdir [concat $::sys_searchpath $::sys_staticpath] {
            ## if pathdir is relative, prepend pwd to it
            set pathdir [file normalize $pathdir]
            # check if the dropped file is in a subdirectory of our PATH            
            if { [string first $pathdir $dir ] == 0 } {
                set found 1
                set obj [string trimleft [file rootname [string range $file [string length $pathdir ] end]] /]
                ::pdwindow::debug "dropping $obj from $pathdir on $::focused_window\n"
                ::dnd_object_create::make_object $mytoplevel $obj
                break
                }
	    }
        if { 2 == $found } {
            set obj [file rootname $file]
            ::pdwindow::debug "dropping $obj on $::focused_window\n"
            ::dnd_object_create::make_object $mytoplevel $obj
        }
#------------------------------------------------------------------------------#       
        }  
        if { 0 == $found } {
            set obj [file rootname $file]
            ::pdwindow::debug "found: $found. dropping $obj on $::focused_window\n"
            ::dnd_object_create::send_filename $mytoplevel $file
        }
#------------------------------------------------------------------------------#     
    }
    return "link"
::pdwindow::debug "67_found: $found.\n"    
}


proc ::dnd_object_create::make_object {w obj} {
    variable x
    variable y
    set posx [expr $x - [winfo rootx $w]]
    set posy [expr $y - [winfo rooty $w]]
    pdsend "$w obj $posx $posy $obj"
    return "dropped"
}

#------------------------------------------------------------------------------#

proc ::dnd_object_create::send_filename {w obj} {
    variable x
    variable y
    set posx [expr $x - [winfo rootx $w]]
    set posy [expr $y - [winfo rooty $w]]
### diese funktion macht dasselbe wie Ctrl+M
### die funktionsdefinition ist in Tcl/pd_connect.tcl, line 50
      ::pd_connect::pdsend "from_ol_dropfile $posx $posy $obj"
### das hier laedt den pd patch "ol_selfdestruct" 
### (damit receivecanvas eine obj message auswirft)
      pdsend "$w obj $posx $posy ol_selfdestruct 5"
### ::pdwindow::debug "$w obj $posx $posy $::focused_window $obj"
    return "dropped"
}
#------------------------------------------------------------------------------#

bind PatchWindow <<Loaded>> {+::dnd_object_create::bind_to_canvas %W}
::tkdnd::drop_target register .pdwindow DND_Files
bind .pdwindow <<Drop:DND_Files>> {::dnd_object_create::open_dropped_files %D}

::pdwindow::post "drop on canvas (modified) loaded\n\n"